import cv2
import numpy as np
import os
import uuid

# Record video for 2 seconds
def record_video():
    cap = cv2.VideoCapture(0)
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    out = cv2.VideoWriter('output.avi', fourcc, 20.0, (640, 480))

    # Record for 2 seconds
    start_time = cv2.getTickCount()
    while True:
        ret, frame = cap.read()
        if not ret:
            break
        out.write(frame)
        current_time = cv2.getTickCount()
        if (current_time - start_time) / cv2.getTickFrequency() > 2:
            break

    cap.release()
    out.release()
    cv2.destroyAllWindows()

# Extract red path from video and display it on screen
def extract_and_display_red_path(video_path):
    cap = cv2.VideoCapture(video_path)
    red_path = []

    while True:
        ret, frame = cap.read()
        if not ret:
            break

        # Convert frame to HSV
        hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)

        # Define lower and upper range for red color
        lower_red = np.array([0, 100, 100])
        upper_red = np.array([10, 255, 255])

        # Threshold the HSV image to get only red colors
        mask = cv2.inRange(hsv, lower_red, upper_red)

        # Find contours in the mask
        contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

        # Append the centroid of the largest contour (red light) to the red path
        if contours:
            largest_contour = max(contours, key=cv2.contourArea)
            M = cv2.moments(largest_contour)
            if M["m00"] != 0:
                centroid_x = int(M["m10"] / M["m00"])
                centroid_y = int(M["m01"] / M["m00"])
                red_path.append((centroid_x, centroid_y))

        # Draw red path on the frame
        for i in range(1, len(red_path)):
            cv2.line(frame, red_path[i-1], red_path[i], (0, 0, 255), thickness=5)

        # Display frame with red path
        cv2.imshow("Red Path", frame)

        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    # Save white path on a black background
    save_red_path_as_image(red_path)
    cap.release()
    cv2.destroyAllWindows()

# Save red path as an image
def save_red_path_as_image(red_path):
    # Create "lumos" folder if it doesn't exist
    os.makedirs("None", exist_ok=True)

    # Generate unique filename
    filename = os.path.join("None", str(uuid.uuid4())[:8] + ".png")

    # Create black background
    black_background = np.zeros((480, 640, 3), dtype=np.uint8)

    # Draw white path on the black background
    for i in range(1, len(red_path)):
        cv2.line(black_background, red_path[i-1], red_path[i], (255, 255, 255), thickness=5)

    # Save image
    cv2.imwrite(filename, black_background)

if __name__ == "__main__":
    # Record video
    record_video()

    # Extract and display red path
    extract_and_display_red_path("output.avi")
